# purple.py

import sys
import string

class SS:   # Stepping Switch
	def __init__(self, fichier ):
		self.table = []
		f = open( fichier , "r")
		while 1:
			ch = f.readline()
			if ch == "": break
			tb = ch.split()
			tb2 = []
			for i in range(1,len(tb)):
				tb2.append( int( tb[i] ))
			self.table.append( tb2 )
		self.position = 0
		self.lg = len (self.table)

	def decipher(self, clair ):
		return self.table[ self.position ][clair]

	def cipher(self, cry ):
		for i in range( len( self.table[ self.position] )):
			if (cry + 1) == self.table[ self.position ][ i ]:
				return i
		print "ERROR: SSW", cry
		sys.exit(1)

	def step(self):
		self.position = self.position + 1
		if self.position == self.lg : self.position = 0

	def setPosition(self, position):
		self.position = position

	def getPosition(self): 
		return self.position

	def getTable(self):
		return self.table

class Plug:
	def __init__(self):
		self.alpha = string.uppercase

	def setAlpha(self, alpha):
		self.alpha = alpha

	def getPos(self,letter):
		indx = self.alpha.find(letter)
		if indx == -1 :
			print "ERROR"
			sys.exit(1)
		return indx

	def isSixes(self,letter):
		indx = self.getPos(letter)
		if indx <= 5 :
			return 1
		else:
			return 0

	def inputDeci(self, indx, flg_sixes ):
		if flg_sixes:
			return self.alpha[ indx ]
		else:
			return self.alpha[ indx + 6 ]	

	def outputDeci(self, letter):
		indx = self.getPos(letter)
		if indx <= 5:
			return indx
		else:
			return indx - 6 

class Type97:
	def __init__(self, debug):
		self.sixes = SS("SSW/switch0.ssw")
		self.twenties = []
		self.twenties.append( SS("SSW/switch1.ssw") )
		self.twenties.append( SS("SSW/switch2.ssw") )
		self.twenties.append( SS("SSW/switch3.ssw") )
		self.plugboard = Plug()
		self.SSW_FAST = 0
		self.SSW_MIDDLE=1
		self.SSW_SLOW = 2
		self.mode = "D"
		self.trigger = 0
		if debug:
			self.DEBUG = 1
		else:
			self.DEBUG = 0

	def printBaseKey(self):
		print self.sixes.getTable()
		print self.twenties[0].getTable()
		print self.twenties[1].getTable()
		print self.twenties[2].getTable()

	def step(self):
		self.sixes.step()
		if self.trigger :
			self.trigger = 0
			self.twenties[ self.SSW_MIDDLE ].step()
		else:
			posSixes = self.sixes.getPosition()
			posMiddle= self.twenties[ self.SSW_MIDDLE ].getPosition()
			if posSixes == 24 :
				if posMiddle == 24:
 					self.twenties[ self.SSW_SLOW ].step()
					self.trigger = 1
				else:
					self.twenties[ self.SSW_MIDDLE ].step()
			else:
				self.twenties[ self.SSW_FAST ].step()			

	def setMotion(self, motion):
		self.SSW_FAST = motion[0] - 1
		self.SSW_MIDDLE=motion[1] - 1
		self.SSW_SLOW = motion[2] - 1

	def setSSW(self,key):
		self.sixes.setPosition( key[0] - 1 )
		for i in range(3):
			self.twenties[i].setPosition( key[i+1] - 1 )

	def getSSW(self):
		tb = []
		tb.append( self.sixes.getPosition() + 1 )
		for i in range(3):
			tb.append( self.twenties[i].getPosition() + 1)
		return tb

	def setPlug(self,alpha):
		self.plugboard.setAlpha( alpha )

	def setMode(self, mode):
		if mode == "D":
			self.mode = "D"
		else:
			self.mode = "E"

	def code(self, cry ):
		if self.mode == "D":
			x = self.decipher( cry )
		else:
			x = self.cipher( cry )
		self.step()
		return x

	def decipher(self, cry ):
		if self.DEBUG: print c.getSSW(), cry
		x = self.plugboard.outputDeci( cry )
		if self.DEBUG: print "===>", x + 1
		if self.plugboard.isSixes( cry ):
			x = self.sixes.decipher(x) - 1
			if self.DEBUG: print "===>", x + 1
			clair = self.plugboard.inputDeci( x, 1 )
		else:
			x = self.twenties[2].decipher(x) - 1
			if self.DEBUG: print "===>", x + 1
			x = self.twenties[1].decipher(x) - 1
			if self.DEBUG: print "===>", x + 1
			x = self.twenties[0].decipher(x) - 1
			if self.DEBUG: print "===>", x + 1
			clair = self.plugboard.inputDeci( x, 0 )
		return clair

	def cipher(self, clair):
		if self.DEBUG: print c.getSSW()
		x = self.plugboard.outputDeci( clair )
		if self.DEBUG: print "===>", x + 1
		if self.plugboard.isSixes( clair ):
			x = self.sixes.cipher( x )
			if self.DEBUG: print "===>", x + 1
			cry = self.plugboard.inputDeci( x, 1 )
		else:
			x = self.twenties[0].cipher(x) 
			if self.DEBUG: print "===>", x + 1
			x = self.twenties[1].cipher(x)
			if self.DEBUG: print "===>", x + 1
			x = self.twenties[2].cipher(x) 
			if self.DEBUG: print "===>", x + 1
			cry = self.plugboard.inputDeci( x, 0 )
		return cry

# ====================== Start of Purple
if len(sys.argv) < 2:

	print "Syntaxe: switches motion plug mode [debug]"
	sys.exit(1);
if len(sys.argv) > 5:
	DEBUG = 1
else:
	DEBUG = 0
les_switches = sys.argv[1].split(":")
for i in range(len(les_switches)):
	les_switches[i] = int( les_switches[i] )
le_motion = sys.argv[2].split(",")
for i in range(len(le_motion)):
	le_motion[i] = int(le_motion[i])
if sys.argv[4] == "D": mode = "D"
else: mode = "E"

c = Type97(DEBUG)
c.setSSW( les_switches )
c.setMotion( le_motion )
c.setPlug( sys.argv[3] )
c.setMode( mode )

while 1:
	ch = sys.stdin.readline()
	if ch == "":
		break
	for i in range(len(ch)):
		if string.uppercase.find( ch[i] ) != -1 :
			sys.stdout.write( c.code( ch[i] ))
			if DEBUG: print
		if ch[i] == "-":
			sys.stdout.write("-")
			c.step()
			if DEBUG: print